function in=forward_selection(allx,y,ME,rel,penter)

%This is a modified version of the stepwisefit function in MATLAB's
%statistics toolbox.
%
%This function performs forward selection assuming weak effect heredity.
%
%Inputs:
% allx: matrix of candidate columns (usually the model matrix excluding the
% intercept
%
% y: response vector
%
% ME: number of factors
%
% rel: relationship matrix for weak effect heredity; obtained from fullmatrix.m 

rel=double(rel);
P=size(allx,2);
in=false(1,P);
eligible=[~in(1:ME) in((ME+1):end)];

while(true)
    [B,PVAL]=stepcalch(allx,y,in,eligible);
    
    nextstep=stepnexth(in,PVAL,penter,eligible);
    
    if nextstep<=ME && nextstep>0
        H=rel(nextstep,2:end);
        eligible=((eligible+H)~=0);
    end
    
    if nextstep==0
        break;
    end
    
    in(nextstep)=~in(nextstep);
        
end


%------------------------------------------------------------------
function [swap,p]=stepnexth(in,PVAL,penter,eligible)
swap=0;
p=NaN;

eligible_not_in=((eligible+~in)==2);
termsout=find(eligible_not_in);

if ~isempty(termsout)
    [pmin,kmin]=min(PVAL(termsout));
    if pmin<penter
        swap=termsout(kmin(1));
        p=pmin;
    end
end


%-----------------------------------------------------------------
function [B,PVAL]=stepcalch(allx,y,in,eligible)

N = length(y);
P = length(in);
X = [ones(N,1) allx(:,in)];
nin = sum(in)+1;
tol = max(N,P+1)*eps(class(allx));
eligible_not_in=((eligible+~in)==2);
num_eligible=sum(eligible_not_in);
not_eligible_not_in=((~eligible+~in)==2);
x = allx(:,eligible_not_in);
sumxsq = sum(x.^2,1);

[Q,R,perm] = qr(X,0);
Rrank = sum(abs(diag(R)) > tol*abs(R(1)));
if Rrank < nin
    R = R(1:Rrank,1:Rrank);
    Q = Q(:,1:Rrank);
    perm = perm(1:Rrank);
end

b = zeros(nin,1);
Qb = Q'*y;
Qb(abs(Qb) < tol*max(abs(Qb))) = 0;
b(perm) = R \ Qb;

r = y - X*b;
dfe = size(X,1)-Rrank;
df0 = Rrank - 1;
SStotal = norm(y-mean(y))^2;
SSresid = norm(r)^2;
perfectyfit = (dfe==0) || (SSresid<tol*SStotal);
if perfectyfit
    SSresid = 0;
    r = 0*r;
end
rmse = sqrt(SSresid./dfe); 
Rinv = R\eye(size(R));
se = zeros(nin,1);
covb(perm,perm) = rmse^2 * (Rinv*Rinv');
se = sqrt(diag(covb));

xr = x - Q*(Q'*x);  
yr = r;             

xx = sum(xr.^2,1);

perfectxfit = (xx<=tol*sumxsq);
if any(perfectxfit)     
    xr(:,perfectxfit) = 0;  
    xx(perfectxfit) = 1;
end
b2 =  (yr'*xr)./xx;  
r2=repmat(yr,1,sum(eligible_not_in))-xr.*repmat(b2,N,1);
df2=max(0,dfe-1);
s2=   sqrt(sum(r2.^2,1)./df2)./sqrt(xx); 

B = zeros(P,1);
B(in) = b(2:end);
B(eligible_not_in)=b2';
B(not_eligible_not_in) = 0;
SE=zeros(P,1);
SE(in)=se(2:end);
SE(eligible_not_in)=s2';

PVAL = zeros(P,1);
tstat = zeros(P,1);
if any(in)
   tval = B(in)./SE(in); 
   ptemp = 2*tcdf(-abs(tval),dfe);
   PVAL(in) = ptemp.*num_eligible;
   tstat(in) = tval;
end
if any(eligible_not_in)
   if dfe>1
      tval =  B(eligible_not_in)./SE(eligible_not_in); 
      ptemp = 2*tcdf(-abs(tval),dfe-1);
   else
      tval = NaN;
      ptemp = NaN;
   end
   PVAL(eligible_not_in) = ptemp.*num_eligible;
   tstat(eligible_not_in) = tval;
end
if any(not_eligible_not_in)
   PVAL(not_eligible_not_in)=Inf;
end
