function [M R labels cols parts] = fullmatrix(X,names,template)

% Function from Wolters, M.A. and Bingham, D. (2011). "Simulated Annealing Model Search for 
% Subset Selection in screening Experiments". Technometrics, 53(3), 225-237. 

% Creates a "full matrix" from a design matrix.  The full matrix contains an 
% intercept column added at left, as well as additional columns added at right 
% in the order requested.  Extra right columns can include:
%       -pairwise differences between design variables;
%       -squared values of the design variables;
%       -arbitrary function with handle 'fhandle' applied to the
%        design variables;
%       -Columns containing all 2-way interactions between variables. 
%
% Valid calls:
%
% M = fullmatrix(X) or M = fullmatrix(X,{},{}) just returns the full matrix
% with intercept and interactions.
%
% [M R] = fullmatrix(X) also returns the relationship matrix R.
%
% [M R labels] = fullmatrix(X,names,template) also returns the labels structure.
%
% [M R labels cols] = fullmatrix(X,names,template) also returns the cols stuct.
% 
% [M R labels cols parts] = fullmatrix(X,names,template) also return the parts
% struct.
%
% An example of names array is {'A' 'B' 'C'}
%
% An example of template array is {'ints' 'diffs' 'quads' 'log' 'sin' 'UserDef'}
%   (where 'UserDef' is the name of a user-defined function)
%
% INPUTS:
% X         -The n-by-p design matrix to be used.
% names     -A p-length cell array of strings giving the names to use for each
%            variable.  If omitted or empty {}, numbers 1:p are used as names.
% template  -A cell array of strings telling the function which sets of extra
%            columns to add, and in what order.  Valid entries are
%               'diffs'   for pairwise differences of all design variables.
%               'quads'   for quadratic terms (cols with squared values of all 
%                         deisgn vars.)
%               'ints'    for 2-way interaction terms among all variables to the
%                         left of 'ints' in the template array.
%               'fname'   where fname is the string name of a function, will
%                         add columns with the corresponding function applied to
%                         all design variables.  Any strings which are not one of
%                         the above 3 will be interpreted as a function handle.
%                         The function must operate elementwise on a matrix.
%            If template is omitted or empty {} then only design variables and
%            interactions are included.  Intercept column is always added.
%
% OUTPUTS:
% M         -The full matrix requested.
% labels    -A structure with cell arrays holding the names of each set of
%            columns, derived from the user-supplied names:
%               labels.all      The complete set of names
%               labels.design   The names of design variables (same as names)
%               labels.diffs    The names of the difference cols, e.g. 'A-B'
%               labels.quads    The names of the quadratic cols, e.g. 'A^2'
%               labels.ints     The names of interaction cols, e.g. 'A*B'
%               labels.fhandle  The names of function cols, e.g. 'fhandle(A)'
% cols      -A structure containing index vectors giving the positions of each
%            subset of columns in the full matrix M.
%               cols.design
%               cols.diffs
%               cols.quads
%               cols.ints
%               cols.fhandle
% parts     -A structure containing the constituent parts of the full matrix,
%            plus the relationship matrix:
%               parts.design    The design matrix (same as X)
%               parts.diffs     The columns of differences
%               parts.diffcombos A 2-by-nchoosek(m,2) matrix giving variable combos 
%                                used in diffs.
%               parts.quads     The columns of squared terms
%               parts.ints      The columns of interactions
%               parts.fname     The columns of function values
%               parts.intcombos A 2-by-? matrix giving var combos used in ints.
%               parts.relmat    The relationship matrix for interactions (see
%                               notes)
%
% NOTES: 
% 1.  The relationship matrix R is a square uint8 matrix with 
%     row and column dimension equal to the number of columns in M. Its rows
%     represent parent variables, columns represent children.  The intercept is
%     considered a parent to itself and the design variables, but not to the other
%     variables.  The intercept must be the fist row/column.  A 1 in the matrix
%     indicates that there is a heredity relationship between the row variable
%     and the col variable.
% 2.  Making R uint8 type rather than logical was found to speed up computations in
%     other functions that use R.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[n m] = size(X);                      %-m = num of design vars

% Set defaults if arguments not supplied, and do error checking on inputs.
% This section does the following: i) make sure a cell array called names
% exists; ii) make sure a cell array called template exists; iii) make sure
% variable ngroups equals the length of template.  If either names or template
% is not supplied or is empty then the default values are supplied.  ngroups is 
% the number of extra blocks of cols to be added.
switch nargin
    case 1
        names = cellstr(num2str((1:m)','%1.0f'))';
        template = {'ints'};
        ngroups = 1;
    case 2
        template = {'ints'};
        ngroups = 1;
        if isempty(names)
            names = cellstr(num2str((1:m)','%1.0f'))';
        end
    case 3
        if isempty(template)
            template = {'ints'};
        end
        if isempty(names)
            names = cellstr(num2str((1:m)','%1.0f'))';
        end
        ngroups = length(template);
    otherwise
        error('Wrong number of arguments')
end

if ~iscell(names) || ~iscell(template)
    error('Names or template argument must be cell arrays')
end

% Determine the ultimate size of M by adding up the columns required to fill in.
% Diffs requires all combinations of design variables; ints requires all combinations
% of variables defined so far; quads and other functions require only one column per
% design variable.
fullsize = m+1;                          %-Plus 1 for intercept.
for i = 1:ngroups
    switch template{i}
        case 'diffs'
            fullsize = fullsize + nchoosek(m,2);
        case 'ints'
            fullsize = fullsize + nchoosek(fullsize-1,2);
        otherwise
            fullsize = fullsize + m;
    end
end

% Create M and R, and labels.all.
M = zeros(n,fullsize);
R = uint8(zeros(fullsize));
labels.all = cell(1,fullsize);

% Update the outputs (M, R, labels, cols, parts) to reflect intercept column and
% design matrix.
M(:,1) = 1;                             %-Add intercept col to M.
M(:,2:m+1) = X;                         %-Add design matrix to M.
R(1,1:m+1) = 1;                         %-Update R.
labels.all{1} = 'icept';
labels.all(2:m+1) = names;         
labels.design = names;
cols.design = 2:(m+1);
parts.design = X;

% Loop through each group specified in template.  For each group, add columns
% to M, update R, and add appropriate members to the output structures.
lec = m+2;                             %-lec = leftmost Empty Column.
for i = 1:ngroups
    switch template{i}
        case 'diffs'
            % Record the columns to use
            ncol = nchoosek(m,2);
            cols.diffs = lec:lec+ncol-1;
            % Form the combinations
            Combos = nchoosek(2:m+1,2)';   %-Each col gives a factor combination.
            a = M(:,Combos(1,:));          %-Matrix a gives 1st X cols for combining.
            b = M(:,Combos(2,:));          %-Matrix b gives 2nd X cols for combining.
            block = a - b;                 %-Create the matrix of difference columns.
            part1 = labels.all(Combos(1,:));
            part2 = labels.all(Combos(2,:));
            % Update M, R, labels, and parts
            M(:,cols.diffs) = block;
            parts.diffs = block;
            parts.diffcombos = Combos;
            labels.diffs = cell(1,ncol);
            for j = 1:ncol
                cc = lec+j-1;           %-Current Column.
                lab = [char(part1(j)) char(45) char(part2(j))];
                labels.all{cc} = lab;
                labels.diffs{j} = lab;
                R(Combos(:,j),cc) = 1;
            end
            lec = lec+ncol;             %-Update leftmost empty column.
        case 'quads'
            % Record the columns to use
            ncol = m;
            cols.quads = lec:lec+ncol-1;
            % Create the new block
            block = M(:,2:m+1).^2;
            % Update M, R, labels, and parts
            M(:,cols.quads) = block;
            parts.quads = block;
            labels.quads = cell(1,ncol);
            for j = 1:ncol
                cc = lec+j-1;           %-Current Column.
                lab = [char(names(j)) char([94 50])];
                labels.all{cc} = lab;
                labels.quads{j} = lab;
                R(j+1,cc) = 1;
            end
            lec = lec+ncol;             %-Update leftmost empty column.
        case 'ints'
            % Record the columns to use
            ncol = nchoosek(lec-2,2);
            cols.ints = lec:lec+ncol-1;
            % Form the combinations
            Combos = nchoosek(2:lec-1,2)'; %-Each col gives a factor combination.
            a = M(:,Combos(1,:));          %-Matrix a gives 1st X cols for combining.
            b = M(:,Combos(2,:));          %-Matrix b gives 2nd X cols for combining.
            block = a.*b;                  %-Create the matrix of interaction columns.
            part1 = labels.all(Combos(1,:));
            part2 = labels.all(Combos(2,:));
            % Update M, R, labels, and parts
            M(:,cols.ints) = block;
            parts.ints = block;
            parts.intcombos = Combos;
            labels.ints = cell(1,ncol);
            for j = 1:ncol
                cc = lec+j-1;           %-Current Column.
                lab = [char(part1(j)) char(42) char(part2(j))];
                labels.all{cc} = lab;
                labels.ints{j} = lab;
                R(Combos(:,j),cc) = 1;
            end
            lec = lec+ncol;             %-Update leftmost empty column.
        otherwise                       %-Treat entry as user-spec'd function
            % Create function.  First check if user fcn is built in or not, and deal
            % with it accordingly.
            funstr = template{i};
            if exist(funstr,'builtin')
                fname = str2func(funstr);
            elseif ismember(funstr,evalin('caller','who'));
                fname = evalin('caller',funstr);
            else
                fname = evalin('base',funstr);
            end
            % Record the columns to use
            ncol = m;
            cols.(funstr) = lec:lec+ncol-1;
            % Create the new block
            block = fname(M(:,2:m+1));
            % Update M, R, labels, and parts
            M(:,cols.(funstr)) = block;
            parts.(funstr) = block;
            labels.(funstr) = cell(1,ncol);
            for j = 1:ncol
                cc = lec+j-1;           %-Current Column.
                lab = [char(funstr) char(40) char(names(j)) char(41)];
                labels.all{cc} = lab;
                labels.(funstr){j} = lab;
                R(j+1,cc) = 1;
            end
            lec = lec+ncol;             %-Update leftmost empty column.
    end
end

end

